﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Printing;
using System.Text;

namespace ch09_components_td
{
    class PageCountPrintController : PreviewPrintController , IComponent
    {

        int pageCount = 0;

        public PageCountPrintController() { }

        public PageCountPrintController(IContainer container)
        {
            container.Add(this);
        }

        public override void OnStartPrint(
          PrintDocument document, PrintEventArgs e)
        {
            base.OnStartPrint(document, e);
            this.pageCount = 0;
        }

        public override System.Drawing.Graphics OnStartPage(
          PrintDocument document, PrintPageEventArgs e)
        {
            // Increment page count
            ++this.pageCount;
            return base.OnStartPage(document, e);
        }

        public int PageCount
        {
            get { return this.pageCount; }
        }

        // Helper method to simplify client code
        public static int GetPageCount(PrintDocument document)
        {
            // Must have a print document to generate page count
            if (document == null)
                throw new ArgumentNullException("PrintDocument must be set.");

            // Subsitute this PrintController to cause a Print to initiate the 
            // count, which means that OnStartPrint and OnStartPage are called 
            // as the PrintDocument prints
            PrintController existingController = document.PrintController;
            PageCountPrintController controller =
              new PageCountPrintController();
            document.PrintController = controller;
            document.Print();
            document.PrintController = existingController;
            return controller.PageCount;
        }


        public event EventHandler Disposed;

        [Browsable(false)]
        [DesignerSerializationVisibility(
            DesignerSerializationVisibility.Hidden)]
        public ISite Site
        {
            get;
            set;
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        ~PageCountPrintController()
        {
            Dispose(false);
        }

        bool isDisposed = false;
        private void Dispose(bool amDisposing)
        {
            if (!isDisposed)
            {
                if (amDisposing)
                {
                    //if managed resources are added,
                    //dispose of them here
                }
                //if unmanaged resources are added,
                //dispose of them here.
                isDisposed = true;
                if (Disposed != null)
                {
                    Disposed(this, EventArgs.Empty);
                }
            }
        }
    }
}


